Re-exports
pub use transforms::Transform;pub use transforms::TransformKind;
Modules
Functions
- hadamard_
transform - Implicitly multiply the argument
xby a Hadamard matrix and scale the results by1 / x.len().sqrt().
pub use transforms::Transform;pub use transforms::TransformKind;x by a Hadamard matrix and scale the results by 1 / x.len().sqrt().