pub enum FailureReason {
DatasetTooSmall,
InsufficientDiversity,
SawInfinity,
}Variants
DatasetTooSmall
This error happens when the dataset contains fewer than the requested number of centers.
InsufficientDiversity
The dataset contains fewer than the requested number of points.
SawInfinity
Infinity was observed (this also happens when a NaN is present in the input data)
Implementations
Sourceimpl FailureReason
impl FailureReason
pub fn is_numerically_recoverable(self) -> bool