Module transforms

Module transforms 

Structs

DoubleHadamard
A Double Hadamard transform that applies the signed Hadamard Transform to a head of the vector and then the tail.
NullTransform
PaddingHadamard
A Hadamard transform that zero pads non-power-of-two dimensions to the next power of two.
RandomRotationlinalg
A distance-perserving transformation from N-dimensions to N-dimensions.

Enums

DoubleHadamardError
NewTransformError
NullTransformErrorflatbuffers
PaddingHadamardError
Errors that may occur while constructing a PaddingHadamard from constituent parts.
RandomRotationErrorlinalg and flatbuffers
TargetDim
Transformations possess the ability to keep dimensionality the same, increase it, or decrease it.
Transform
TransformErrorflatbuffers
TransformFailed
TransformKind