TransformKind

Enum TransformKind 

#[non_exhaustive]
pub enum TransformKind { PaddingHadamard { target_dim: TargetDim, }, DoubleHadamard { target_dim: TargetDim, }, Null, RandomRotation { target_dim: TargetDim, }, }

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


PaddingHadamard

Use a Hadamard transform

HDx / sqrt(n)

where

  • H is an (implicit) Hadamard matrix
  • D is a diagonal matrix with +/-1 on the diagonal.
  • x is the input vector.
  • n is the number of rows in x.

Unlike Self::RandomRotation, this method does not require matrix-vector multiplication and is therefore much faster for high-dimensional vectors.

The Hadamard multiplication requires dimensions to be a power of two. Internally, this method will pad x with zeros up to the next power of two and transform the result.

Fields

target_dim: TargetDim


DoubleHadamard

Use a Double Hadamard transform, which applies two Hadamard transformations in sequence; first to the head of the vector and then to the tail.

This approach does not have any requirement on the input dimension to apply the distance preserving transformation using Hadamard multiplication, unlike PaddingHadamard.

Empirically, this approach seems to give better recall performance than applying PaddingHadamard and sampling down when self.output_dim() == self.dim() and the dimension is not a power of two.

See DoubleHadamard for the implementation details.

Fields

target_dim: TargetDim


Null

A naive transform that copies source into destination.



RandomRotation

Available on crate feature linalg only.

Use a full-dimensional, randomly sampled orthogonal matrix to transform vectors.

Transformation involves matrix multiplication and may be slow for high-dimensional vectors.

Fields

target_dim: TargetDim