PaddingHadamard

Struct PaddingHadamard 

pub struct PaddingHadamard<A>
where A: Allocator,
{ /* private fields */ }
A Hadamard transform that zero pads non-power-of-two dimensions to the next power of two.

This struct performs the transformation

HDx / sqrt(n)

where

  • H is a Hadamard Matrix
  • D is a diagonal matrix with diagonal entries in {-1, +1}.
  • x is the vector to transform, zero padded to have a length that is a multiple of two.
  • n is the output-dimension.

Implementations

Source

impl<A> PaddingHadamard<A>
where A: Allocator,

pub fn new<R>( dim: NonZeroUsize, target: TargetDim, rng: &mut R, allocator: A, ) -> Result<Self, AllocatorError>
where R: Rng + ?Sized,

Construct a new PaddingHadamard that transforms input vectors of dimension dim.

The parameter rng is used to randomly initialize the diagonal matrix portion of the transform.

The following dimensionalities will be configured depending on the value of target:

  • TargetDim::Same
    • self.input_dim() == dim.get()
    • self.output_dim() == dim.get()
  • TargetDim::Natural
    • self.input_dim() == dim.get()
    • self.output_dim() == dim.get().next_power_of_two()
  • TargetDim::Override
    • self.input_dim() == dim.get()
    • self.output_dim(): The value provided by the override.

Subsampling occurs if self.output_dim() is not a power of two and greater-than or equal to self.input_dim().

pub fn try_from_parts( signs: Poly<[u32], A>, padded_dim: usize, subsample: Option<Poly<[u32], A>>, ) -> Result<Self, PaddingHadamardError>

Construct Self from constituent parts. This validates that the necessary invariants hold for the constituent parts, returning an error if they do not.

pub fn input_dim(&self) -> usize

Return the input dimension for the transformation.

pub fn output_dim(&self) -> usize

Return the output dimension for the transformation.

pub fn preserves_norms(&self) -> bool

Return whether or not the transform preserves norms.

For this transform, norms are not preserved when the output dimensionality is not a power of two greater than or equal to the input dimensionality.

pub fn transform_into( &self, dst: &mut [f32], src: &[f32], allocator: ScopedAllocator<'_>, ) -> Result<(), TransformFailed>

Perform the transformation of the src vector into the dst vector.

Errors

Returns an error if

  • src.len() != self.input_dim().
  • dst.len() != self.output_dim().