pub struct RandomRotation { /* private fields */ }linalg only.Expand description
A distance-perserving transformation from N-dimensions to N-dimensions.
This struct materializes a full NxN transformation matrix and mechanically applies
transformations via matrix multiplication.
Implementations
Sourceimpl RandomRotation
impl RandomRotation
Sourcepub fn new<R>(dim: NonZeroUsize, target_dim: TargetDim, rng: &mut R) -> Self
pub fn new<R>(dim: NonZeroUsize, target_dim: TargetDim, rng: &mut R) -> Self
Construct a new RandomRotation that transforms input vectors of dimension dim.
The parameter rng is used to randomly sample the transformation matrix.
The following dimensionalities will be configured depending on the value of target:
TargetDim::Sameself.input_dim() == dim.get()self.output_dim() == dim.get()
TargetDim::Naturalself.input_dim() == dim.get()self.output_dim() == dim.get()
TargetDim::Overrideself.input_dim() == dim.get()self.output_dim(): The value provided by the override.
Sub-sampling occurs if self.output_dim() is less than self.input_dim().
Sourcepub fn output_dim(&self) -> usize
pub fn output_dim(&self) -> usize
Return the output dimension for the transformation.
Sourcepub fn preserves_norms(&self) -> bool
pub fn preserves_norms(&self) -> bool
Return whether or not the transform preserves norms.
For this transform, norms are not preserved when the output dimensionality is less than the input dimensionality.
Sourcepub fn transform_into(
&self,
dst: &mut [f32],
src: &[f32],
) -> Result<(), TransformFailed>
pub fn transform_into( &self, dst: &mut [f32], src: &[f32], ) -> Result<(), TransformFailed>
Perform the transformation of the src vector into the dst vector.
Errors
Returns an error if
src.len() != self.input_dim().dst.len() != self.output_dim().