Module alloc

Module alloc 

Macros

poly

Structs

AlignedAllocator
An AllocatorCore that allocates memory aligned to at least a specified alignment.
AllocatorError
Indicate that an allocation error has occurred.
BumpAllocator
An AllocatorCore that pre-allocates a large buffer of memory and then satisfies allocation requests from that buffer.
GlobalAllocator
A handle to Rust’s global allocator. This type does not support allocations of size 0.
NotPowerOfTwo
Poly
An owning pointer type like std::Box that supports custom allocators.
ScopedAllocator
A dynamic wrapper around an AllocatorCore that provides the guarantee that all allocated object are tied to a given scope.

Enums

CompoundError
Error type returned from Poly::new_with.

Traits

Allocator
A dynamic memory allocator for use with crate::alloc::Poly.
AllocatorCore
A dynamic memory allocator for use with crate::alloc::Poly.
TrustedIter
A local marker type for iterators with a trusted length.