AlignedAllocator

Struct AlignedAllocator 

pub struct AlignedAllocator { /* private fields */ }
An AllocatorCore that allocates memory aligned to at least a specified alignment.

This can be useful for large allocations that need a predictable base alignment.

Implementations

Source

impl AlignedAllocator

pub const fn new(alignment: PowerOfTwo) -> Self

Construct a new allocator that uses the given alignment.

pub const fn alignment(&self) -> usize