AllocatorError

Struct AllocatorError 

pub struct AllocatorError;
Indicate that an allocation error has occurred.

This type is limited in what it can contain because additional context inevitably requires more memory allocation, which is what we’re trying to avoid.