BumpAllocator

Struct BumpAllocator 

pub struct BumpAllocator { /* private fields */ }
An AllocatorCore that pre-allocates a large buffer of memory and then satisfies allocation requests from that buffer.

Note that the memory for allocations made through this allocator and its clones will not be freed until all clones have been dropped.

Memory allocation through this page is thread safe.

Implementations

Source

impl BumpAllocator

pub fn new( capacity: usize, alignment: PowerOfTwo, ) -> Result<Self, AllocatorError>

Construct a new BumpAllocator with room for capacity bytes. The base pointer for the allocator will be aligned to at least alignment bytes.

Returns an error if alignment is not a power of two or if an error occurs during memory allocation.

pub fn capacity(&self) -> usize

Return the capacity this allocator was created with.

pub fn as_ptr(&self) -> *const u8

Return a pointer to the base of the buffer behind this allocator.