ScopedAllocator

Struct ScopedAllocator 

pub struct ScopedAllocator<'a> { /* private fields */ }
A dynamic wrapper around an AllocatorCore that provides the guarantee that all allocated object are tied to a given scope.

Additionally, this can allow the use of an allocator that is not Clone in contexts where a clonable allocator is needed (provided the scoping limitations are acceptable).

Implementations

Source

impl<'a> ScopedAllocator<'a>

pub const fn new<T>(allocator: &'a T) -> Self
where T: AllocatorCore + Debug,

Construct a new ScopedAllocator around the provided allocator.

Source

impl ScopedAllocator<'static>

pub const fn global() -> Self

A convenience method for construcing a ScopedAllocator around the GlobalAllocator for cases where a more specialized allocator is not needed.