Utilities for working with packed 1-bit to 8-bit integers (inclusive).
Modules
- distances
- Low-level functions
Structs
- Binary
- A 1-bit binary quantization mapping
-1to0and1to1. - BitSlice
Base - A generalized representation for packed small bit integer encodings over a contiguous span of memory.
- BitTranspose
- A layout specialized for performing multi-bit operations with 1-bit scalar quantization.
- Construction
Error - Dense
- The identity permutation strategy.
- Dynamic
- A type for dynamically sized slices.
- Encoding
Error - Index
OutOf Bounds - MutSlice
Ptr - A mutable pointer with an associated lifetime.
- Slice
Ptr - A constant pointer with an associated lifetime.
- Static
- A type for statically sized slices (slices with a length known at compile-time).
- Unsigned
- Storage unsigned integers in slices.
Enums
Traits
- AsMut
Ptr - Obtain a mutable base pointer for a slice of data.
- AsPtr
- Obtain a constant base pointer for a slice of data.
- Length
- A trait allowing a type to yield a value.
- Permutation
Strategy - A enable the dimensions within a BitSlice to be permuted in an arbitrary way.
- Representation
- Representation of
NBITSbit numbers in the associated domain.
Type Aliases
- BitSlice
- The layout for
N-bit integers that references a raw underlying slice. - Boxed
BitSlice - The layout for
N-bit integers that own the underlying slice. - MutBit
Slice - The layout for
N-bit integers that mutable references a raw underlying slice.