pub struct Unsigned;Expand description
Storage unsigned integers in slices.
For a bit count of NBITS, the Unsigned type can store unsigned integers in
the range [0, 2^NBITS - 1].
Implementations
Sourceimpl Unsigned
impl Unsigned
Sourcepub const fn domain_const<const NBITS: usize>() -> RangeInclusive<i64>
pub const fn domain_const<const NBITS: usize>() -> RangeInclusive<i64>
Return the dynamic range of an Unsigned encoding for NBITS.