pub unsafe trait AsPtr {
type Type;
// Required method
fn as_ptr(&self) -> *const Self::Type;
}Expand description
Obtain a constant base pointer for a slice of data.
Safety
The returned pointer must never be null!
pub unsafe trait AsPtr {
type Type;
// Required method
fn as_ptr(&self) -> *const Self::Type;
}Obtain a constant base pointer for a slice of data.
The returned pointer must never be null!