pub struct UnequalLengths;Expand description
A marker type that indicates a distance computation failed because the arguments had unequal lengths.
This struct intentionally is a zero-sized type to allow return paths to be as efficient as possible.
Implementations