MathematicalResult

Type Alias MathematicalResult 

pub type MathematicalResult<T> = Result<MathematicalValue<T>, UnequalLengths>;
Upcoming return type for distance functions to allow graceful failure instead of panicking when incorrect dimensions are provided.

Aliased Type

pub enum MathematicalResult<T> {
    Ok(MathematicalValue<T>),
    Err(UnequalLengths),
}

Variants

1.0.0

Ok(MathematicalValue<T>)

Contains the success value

1.0.0

Err(UnequalLengths)

Contains the error value