pub type Result<T> = Result<T, UnequalLengths>;Expand description
Upcoming return type for distance functions to allow graceful failure instead of panicking when incorrect dimensions are provided.
Aliased Type
pub enum Result<T> {
Ok(T),
Err(UnequalLengths),
}