#[non_exhaustive]pub enum Parallelism {
Sequential,
Rayon,
}Expand description
Selector for the parallelization strategy used by some algorithms.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Sequential
Use single-threaded execution.
Rayon
Available on crate feature
rayon only.Use Rayon based parallelism in the dynamically scoped Rayon thread pool.