Module error

Module error 

Formatting utilities for error chains.

Structs

InlineError
An implementation of Box<dyn std::error::Error> that stores the error payload inline, avoiding dynamic memory allocation. This has several practical drawbacks:

Enums

Infallible

Functions

format
Format the entire error chain for err by first calling err.to_string() and then by walking the error’s source tree.