pub struct InlineError<const N: usize = 16> { /* private fields */ }Expand description
An implementation of Box<dyn std::error::Error> that stores the error payload inline,
avoiding dynamic memory allocation. This has several practical drawbacks:
- The size of the error payload must be at most
Nbytes. - The alignment of the error payload must be at most 8 bytes.
Both of these contraints are verified using post-monomorphization errors.
Example
use diskann_quantization::error::InlineError;
let base_error = u32::try_from(u64::MAX).unwrap_err();
let mut error = InlineError::<8>::new(base_error);
assert_eq!(error.to_string(), base_error.to_string());
// Change the dynamic type of the contained error.
error = InlineError::new(Box::new(base_error));
assert_eq!(error.to_string(), base_error.to_string());Implementations
Sourceimpl<const N: usize> InlineError<N>
impl<const N: usize> InlineError<N>