Module slice

Module slice 

Structs

Slice
A wrapper for a traditional Rust slice that provides the addition of arbitrary metadata.

Enums

NotCanonical

Type Aliases

PolySlice
An owning slice and associated metadata.
SliceMut
A mutable reference to a slice and associated metadata.
SliceRef
A reference to a slice and associated metadata.