MinMaxMeta

Struct MinMaxMeta 

pub struct MinMaxMeta<const NBITS: usize> { /* private fields */ }
Metadata for MinMax quantized multi-vectors.

Stores the intrinsic dimension (output dimension after transform) which is needed to interpret each row of the quantized data. The row stride in bytes is computed as [Data::<NBITS>::canonical_bytes(intrinsic_dim)].

Implementations

Source

impl<const NBITS: usize> MinMaxMeta<NBITS>
where Unsigned: Representation<NBITS>,

pub fn new(nrows: usize, intrinsic_dim: usize) -> Self

Creates new MinMax metadata with the given number of rows and intrinsic dimension.

pub fn intrinsic_dim(&self) -> usize

Returns the intrinsic_dim

pub fn ncols(&self) -> usize

Returns the number of bytes from a canonical repr of a minmax quantized vector, see minmax::Data::canonical_bytes