pub struct MinMaxQuantizer { /* private fields */ }Expand description
Recall that from the module-level documentation, MinMaxQuantizer, quantizes X
into n bit vectors as follows -
X' = round((X - s) * (2^n - 1) / c).clamp(0, 2^n - 1))where s is a shift value and c is a scaling parameter computed from the range of values.
For most bit widths (>1), given a positive scaling parameter grid_scale : f32,
these are computed as:
- m = (max_i X[i] + min_i X[i]) / 2.0
- w = max_i X[i] - min_i X[i]
- s = m - w * grid_scale
- c = 2 * w * grid_scale
where `grid_scale` is an input to the quantizer.For 1-bit quantization, to avoid outliers, s and c are derived differently:
- Values are first split into two groups: those below and above the mean.
sis the average of values below the mean.cis the difference between the average of values above the mean ands.
See MinMaxCompensation for notation.
We have then that
X = X' * (c / (2^n - 1)) + s
-------------- -
| |
ax bxImplementations
Sourceimpl MinMaxQuantizer
impl MinMaxQuantizer
Sourcepub fn new(
transform: Transform<GlobalAllocator>,
grid_scale: Positive<f32>,
) -> Self
pub fn new( transform: Transform<GlobalAllocator>, grid_scale: Positive<f32>, ) -> Self
Instantiates a new quantizer with specific transform.
Sourcepub fn output_dim(&self) -> usize
pub fn output_dim(&self) -> usize
Output dimension of vectors after applying transform.
Output storage vectors should use this dimension instead of self.dim() because
in general, the output dim may be different from the input dimension.