pub struct BlockTransposedMut<'a, T: Copy, const GROUP: usize, const PACK: usize = 1> { /* private fields */ }Expand description
A mutable view of a block-transposed matrix.
Created by BlockTransposed::as_view_mut.
Implementations
Sourceimpl<'a, T: Copy, const GROUP: usize, const PACK: usize> BlockTransposedMut<'a, T, GROUP, PACK>
impl<'a, T: Copy, const GROUP: usize, const PACK: usize> BlockTransposedMut<'a, T, GROUP, PACK>
Sourcepub fn as_view(&self) -> BlockTransposedRef<'_, T, GROUP, PACK>
pub fn as_view(&self) -> BlockTransposedRef<'_, T, GROUP, PACK>
Borrow as an immutable BlockTransposedRef.
Sourcepub fn padded_ncols(&self) -> usize
pub fn padded_ncols(&self) -> usize
Sourcepub fn full_blocks(&self) -> usize
pub fn full_blocks(&self) -> usize
Sourcepub fn num_blocks(&self) -> usize
pub fn num_blocks(&self) -> usize
Sourcepub unsafe fn block_ptr_unchecked(&self, block: usize) -> *const T
pub unsafe fn block_ptr_unchecked(&self, block: usize) -> *const T
Sourcepub fn block(&self, block: usize) -> MatrixView<'_, T>
pub fn block(&self, block: usize) -> MatrixView<'_, T>
Sourcepub fn remainder_block(&self) -> Option<MatrixView<'_, T>>
pub fn remainder_block(&self) -> Option<MatrixView<'_, T>>
Sourcepub fn get_element(&self, row: usize, col: usize) -> T
pub fn get_element(&self, row: usize, col: usize) -> T
Sourcepub const fn group_size(&self) -> usize
pub const fn group_size(&self) -> usize
Group size (blocking factor GROUP).
Sourcepub const fn const_group_size() -> usize
pub const fn const_group_size() -> usize
Group size as const function on the type.
Sourcepub fn get_row(&self, i: usize) -> Option<Row<'_, T, GROUP, PACK>>
pub fn get_row(&self, i: usize) -> Option<Row<'_, T, GROUP, PACK>>
Get an immutable row view, or None if i is out of bounds.
Sourcepub fn as_mut_slice(&mut self) -> &mut [T]
pub fn as_mut_slice(&mut self) -> &mut [T]
Return the backing data as a mutable slice.
The returned slice has storage_len() elements (including all padding).
Sourcepub fn block_mut(&mut self, block: usize) -> MutMatrixView<'_, T>
pub fn block_mut(&mut self, block: usize) -> MutMatrixView<'_, T>
Sourcepub fn remainder_block_mut(&mut self) -> Option<MutMatrixView<'_, T>>
pub fn remainder_block_mut(&mut self) -> Option<MutMatrixView<'_, T>>
Return a mutable view over the remainder block, or None if there is no
remainder.
Sourcepub fn get_row_mut(&mut self, i: usize) -> Option<RowMut<'_, T, GROUP, PACK>>
pub fn get_row_mut(&mut self, i: usize) -> Option<RowMut<'_, T, GROUP, PACK>>
Get a mutable row view, or None if i is out of bounds.