pub struct Row<'a, T, const GROUP: usize, const PACK: usize = 1> { /* private fields */ }Expand description
An immutable view of a single logical row in a block-transposed matrix.
Because the elements of a logical row are strided (not contiguous), this struct provides indexed access and iteration over the row’s elements.
Implementations