MaxSim

Struct MaxSim 

pub struct MaxSim<'a> { /* private fields */ }
Computes per-query-vector maximum similarities to document vectors.

For each query vector qᵢ, finds the maximum similarity (minimum negated inner product) to any document vector:

scores[i] = minⱼ -IP(qᵢ, dⱼ)

Implements DistanceFnMut for various matrix types (e.g., MatRef<Standard<f32>>).

Usage

  • Create with MaxSim::new, providing a mutable scores buffer.
  • Call DistanceFnMut::evaluate with query and document matrices.
  • Read results from the scores buffer.

Implementations

Source

impl<'a> MaxSim<'a>

pub fn new(scores: &'a mut [f32]) -> Result<Self, MaxSimError>

Creates a new MaxSim with the provided scores buffer.

Errors

Returns an error if scores is empty.

pub fn size(&self) -> usize

Returns the number of score slots in the buffer.

pub fn get(&self, i: usize) -> Result<f32, MaxSimError>

Returns the score at index i.

pub fn set(&mut self, i: usize, x: f32) -> Result<(), MaxSimError>

Sets the score at index i.

pub fn scores_mut(&mut self) -> &mut [f32]

Returns a mutable reference to the internal buffer of scores.

This is useful for implementations external to crate as well as optimized implementations to access the buffer if needed.