Mat

Struct Mat 

pub struct Mat<T: ReprOwned> { /* private fields */ }
An owning matrix that manages its own memory.

The matrix stores raw bytes interpreted according to representation type T. Memory is automatically deallocated when the matrix is dropped.

Implementations

Source

impl<T: ReprOwned> Mat<T>

pub fn new<U>(repr: T, init: U) -> Result<Self, <T as NewOwned<U>>::Error>
where T: NewOwned<U>,

Create a new matrix using init as the initializer.

pub fn num_vectors(&self) -> usize

Returns the number of rows (vectors) in the matrix.

pub fn repr(&self) -> &T

Returns a reference to the underlying representation.

pub fn get_row(&self, i: usize) -> Option<T::Row<'_>>

Returns the ith row if i < self.num_vectors().

pub fn get_row_mut(&mut self, i: usize) -> Option<T::RowMut<'_>>

Returns the ith mutable row if i < self.num_vectors().

pub fn as_view(&self) -> MatRef<'_, T>

Returns an immutable view of the matrix.

pub fn as_view_mut(&mut self) -> MatMut<'_, T>

Returns a mutable view of the matrix.

pub fn rows(&self) -> Rows<'_, T>

Returns an iterator over immutable row references.

pub fn rows_mut(&mut self) -> RowsMut<'_, T>

Returns an iterator over mutable row references.

pub fn as_raw_ptr(&self) -> *const u8

Return the base pointer for the Mat.

Source

impl<T: Copy> Mat<Standard<T>>

pub fn vector_dim(&self) -> usize

Returns the raw dimension (columns) of the vectors in the matrix.