MatMut

Struct MatMut 

pub struct MatMut<'a, T: ReprMut> { /* private fields */ }
A mutable borrowed view of a matrix.

Provides read-write access to matrix data without ownership.

Type Parameter

  • T: A ReprMut implementation defining the row layout.

Access

Implementations

Source

impl<'a, T: ReprMut> MatMut<'a, T>

pub fn new<U>(repr: T, data: &'a mut [U]) -> Result<Self, T::Error>
where T: NewMut<U>,

Construct a new MatMut over data.

pub fn num_vectors(&self) -> usize

Returns the number of rows (vectors) in the matrix.

pub fn repr(&self) -> &T

Returns a reference to the underlying representation.

pub fn get_row(&self, i: usize) -> Option<T::Row<'_>>

Returns an immutable reference to the i-th row, or None if out of bounds.

pub fn get_row_mut(&mut self, i: usize) -> Option<T::RowMut<'_>>

Returns a mutable reference to the i-th row, or None if out of bounds.

pub fn as_view(&self) -> MatRef<'_, T>

Reborrows as an immutable MatRef.

pub fn rows(&self) -> Rows<'_, T>

Returns an iterator over immutable row references.

pub fn rows_mut(&mut self) -> RowsMut<'_, T>

Returns an iterator over mutable row references.

pub fn to_owned(&self) -> Mat<T>
where T: NewCloned,

Return a Mat with the same contents as self.

pub unsafe fn from_raw_parts(repr: T, ptr: NonNull<u8>) -> Self

Construct a new MatMut over the raw pointer and representation without performing any validity checks.

Safety

Argument ptr must point to memory compatible with Repr::layout.

pub fn as_raw_ptr(&self) -> *const u8

Return the base pointer for the MatMut.

Source

impl<'a, T: Copy> MatMut<'a, Standard<T>>

pub fn vector_dim(&self) -> usize

Returns the raw dimension (columns) of the vectors in the matrix.