MatRef

Struct MatRef 

pub struct MatRef<'a, T: Repr> { /* private fields */ }
An immutable borrowed view of a matrix.

Provides read-only access to matrix data without ownership. Implements Copy and can be freely cloned.

Type Parameter

  • T: A Repr implementation defining the row layout.

Access

  • get_row: Get an immutable row by index.
  • rows: Iterate over all rows.

Implementations

Source

impl<'a, T: Repr> MatRef<'a, T>

pub fn new<U>(repr: T, data: &'a [U]) -> Result<Self, T::Error>
where T: NewRef<U>,

Construct a new MatRef over data.

pub fn num_vectors(&self) -> usize

Returns the number of rows (vectors) in the matrix.

pub fn repr(&self) -> &T

Returns a reference to the underlying representation.

pub fn get_row(&self, i: usize) -> Option<T::Row<'_>>

Returns an immutable reference to the i-th row, or None if out of bounds.

pub fn rows(&self) -> Rows<'_, T>

Returns an iterator over immutable row references.

pub fn to_owned(&self) -> Mat<T>
where T: NewCloned,

Return a Mat with the same contents as self.

pub unsafe fn from_raw_parts(repr: T, ptr: NonNull<u8>) -> Self

Construct a new MatRef over the raw pointer and representation without performing any validity checks.

Safety

Argument ptr must point to memory compatible with Repr::layout and pass any validity checks required by T.

pub fn as_raw_ptr(&self) -> *const u8

Return the base pointer for the MatRef.

Source

impl<'a, T: Copy> MatRef<'a, Standard<T>>

pub fn vector_dim(&self) -> usize

Returns the raw dimension (columns) of the vectors in the matrix.