Standard

Struct Standard 

pub struct Standard<T> { /* private fields */ }
Metadata for dense row-major matrices of Copy types.

Rows are stored contiguously as &[T] slices. This is the default representation type for standard floating-point multi-vectors.

Row Types

  • Row<'a>: &'a [T]
  • RowMut<'a>: &'a mut [T]

Implementations

Source

impl<T: Copy> Standard<T>

pub fn new(nrows: usize, ncols: usize) -> Result<Self, Overflow>

Create a new Standard for data of type T.

Successful construction requires:

  • The total number of elements determined by nrows * ncols does not exceed usize::MAX.
  • The total memory footprint defined by ncols * nrows * size_of::<T>() does not exceed isize::MAX.
pub fn num_elements(&self) -> usize

Returns the number of total elements (rows x cols) in this matrix.