pub struct Positive<T>(/* private fields */)
where
T: PartialOrd + Default + Debug;Expand description
A number type that must be greater than zero.
Implementations
Sourceimpl<T> Positive<T>
impl<T> Positive<T>
Sourcepub fn new(value: T) -> Result<Self, NotPositiveError<T>>
pub fn new(value: T) -> Result<Self, NotPositiveError<T>>
Create a new Positive if the given value is greater than 0 (T::default());
Sourcepub const unsafe fn new_unchecked(value: T) -> Self
pub const unsafe fn new_unchecked(value: T) -> Self
Create a new Positive without checking whether the value is greater than 0.
Safety
The value must be greater than T::default().
Sourcepub fn into_inner(self) -> T
pub fn into_inner(self) -> T
Consume self and return the inner value.