Module product

Module product 

Product quantization training and compression.

Modules

train

Structs

BasicTableBase
A basic PQ table that stores the pivot table in the following dense, row-major form:
TransposedTable
A PQ table that stores the pivots for each chunk in a miniture block-transpose to facilitate faster compression. The exact layout is not documented (as it is for the BasicTable) because it may be subject to change.

Enums

TableCompressionError

Type Aliases

BasicTable
A BasicTableBase that owns its contents.
BasicTableView
A BasicTableBase that references its contents. Construction of such a table will not result in a memory allocation.