pub struct BasicTableBase<T, U>{ /* private fields */ }Expand description
A basic PQ table that stores the pivot table in the following dense, row-major form:
| -- chunk 0 -- | -- chunk 1 -- | -- chunk 2 -- | .... | -- chunk N-1 -- |
+------------------------------------------------------------------------+
pivot 0 | c000 c001 ... | c010 c011 ... | c020 c021 ... | .... | ... |
pivot 1 | c100 c101 ... | c110 c111 ... | c120 c121 ... | .... | ... |
... | ... | ... | ... | .... | ... |
pivot K | cK00 cK01 ... | cK10 cK11 ... | cK20 cK21 ... | .... | ... |The member offsets describes the number of dimensions of each chunk.
Invariants
offsets.dim() == pivots.nrows(): The dimensionality of the two must agree.
Implementations
Sourceimpl<T, U> BasicTableBase<T, U>
impl<T, U> BasicTableBase<T, U>
Sourcepub fn new(
pivots: MatrixBase<T>,
offsets: ChunkOffsetsBase<U>,
) -> Result<Self, BasicTableError>
pub fn new( pivots: MatrixBase<T>, offsets: ChunkOffsetsBase<U>, ) -> Result<Self, BasicTableError>
Construct a new BasicTableBase over the pivot table and offsets.
Error
Returns an error if pivots.ncols() != offsets.dim() or if pivots.nrows() == 0.
Sourcepub fn view_pivots(&self) -> MatrixView<'_, f32>
pub fn view_pivots(&self) -> MatrixView<'_, f32>
Return a view over the pivot table.
Sourcepub fn view_offsets(&self) -> ChunkOffsetsView<'_>
pub fn view_offsets(&self) -> ChunkOffsetsView<'_>
Return a view over the schema offsets.