TransposedTable

Struct TransposedTable 

pub struct TransposedTable { /* private fields */ }
A PQ table that stores the pivots for each chunk in a miniture block-transpose to facilitate faster compression. The exact layout is not documented (as it is for the BasicTable) because it may be subject to change.

The advantage of this table over the BasicTable is that this table uses a more hardware friendly layout for the pivots, meaning that compression (particularly batch compression) is much faster than the basic table, at the cost of a slightly higher memory footprint.

Invariants (Dev Docs)

  • pivots.len() == schema.len(): The number of PQ chunks must agree.
  • pivots[i].dimension() == schema.at(i).len() for all inbounds i.
  • pivots[i].total() == ncenters for all inbounds i.
  • largest = max(schema.at(i).len() for i).

Implementations

Source

impl TransposedTable

pub fn from_parts( pivots: MatrixView<'_, f32>, offsets: ChunkOffsets, ) -> Result<Self, TransposedTableError>

Construct a new TransposedTable from raw parts.

Error

Returns an error if

  • pivots.ncols() != offsets.dim(): Pivots must have the dimensionality expected by the offsets.

  • pivots.nrows() == 0: The pivot table cannot be empty.

pub fn ncenters(&self) -> usize

Return the number of pivots in each PQ chunk.

pub fn nchunks(&self) -> usize

Return the number of PQ chunks.

pub fn dim(&self) -> usize

Return the dimensionality of the full-precision vectors associated with this table.

pub fn view_offsets(&self) -> ChunkOffsetsView<'_>

Return a view over the schema offsets.

pub fn compress_batch<T, F, DelegateError>( &self, data: MatrixView<'_, T>, compression_delegate: F, ) -> Result<(), CompressError<DelegateError>>
where T: Copy + Into<f32>, F: FnMut(RowChunk, CompressionResult) -> Result<(), DelegateError>, DelegateError: Error,

Perform PQ compression on the dataset by mapping each chunk in data to its nearest neighbor in the corresponding entry in chunks.

The index of the nearest neighbor is provided to compression_delegate along with its corresponding row in data and chunk index.

Calls to compression_delegate may occur in any order.

Visitor will be invoked for all rows in 0..data.nrows() and all chunks in 0..schema.len().

Panics

Panics under the following conditions:

  • data.cols() != self.dim(): The number of columns in the source dataset must match the number of dimensions expected by the schema.
pub fn process_into<T>(&self, query: &[f32], partials: MutMatrixView<'_, f32>)
where T: ProcessInto,

Compute the operation defined by T for each chunk in the query on all corresponding pivots for that chunk, storing the result in the output matrix.

For example, this can be used to compute squared L2 distances between chunks of the query and the pivot table to create a fast run-time lookup table for these distances.

This is currently implemented for the following operation types T:

  • quantization::distances::SquaredL2
  • quantization::distances::InnerProduct
Arguments
  • query: The query slice to process. Must have length self.dim().

  • partials: Output matrix for the partial results. The result of the computation of chunk i against pivot j will be stored into pivots[(i, j)].

    Must have nrows = self.nchunks() and ncols = self.ncenters().

Panics

Panics if:

  • query.len() != self.dim().
  • partisl.nrows() != self.nchunks().
  • partisl.ncols() != self.ncenters().