pub trait RngBuilder<T> {
type Rng: Rng + 'static;
// Required method
fn build_rng(&self, mixin: T) -> Self::Rng;
}Expand description
Creation of random number generator in potentially parallelized applications.
pub trait RngBuilder<T> {
type Rng: Rng + 'static;
// Required method
fn build_rng(&self, mixin: T) -> Self::Rng;
}Creation of random number generator in potentially parallelized applications.