CompensatedIP

Struct CompensatedIP 

pub struct CompensatedIP { /* private fields */ }
A DistanceFunction containing scaling parameters to enable distance the SquaredL2 distance function over CompensatedVectors belonging to the same quantization space.

Implementations

Source

impl CompensatedIP

pub fn new(scale_squared: f32, shift_square_norm: f32) -> Self

Construct a new CompensatedIP with the given scaling factor and shift norm.