pub enum PreScale {
None,
Some(Positive<f32>),
ReciprocalMeanNorm,
}Expand description
Pre-scaling selector for spherical quantization training. Pre-scaling adjusts the dynamic range of the data (usually decreasing it uniformly) to keep the correction terms within the range expressible by 16-bit floating point numbers.
Variants
None
Do not use any pre-scaling.
Some(Positive<f32>)
Pre-scale all data by the specified amount.
ReciprocalMeanNorm
Heuristically estimate a pre-scaling parameter by using the inverse approximate mean norm. This will nearly normalize in-distribution vectors.