pub enum CompressionError {
NotCanonical(InlineError<16>),
CompressionError(CompressionError),
}Expand description
Errors that can occur during data compression
Variants
NotCanonical(InlineError<16>)
The input buffer did not have the expected layout. This is an input error.
CompressionError(CompressionError)
Forward any error that occurs during the compression process.
See quantizer::CompressionError for the complete list.