pub enum DistanceError {
QueryReify(InlineError<24>),
XReify(InlineError<16>),
UnequalLengths(UnequalLengths),
}Expand description
Errors that can occur while perfoming distance cacluations on opaque vectors.
Variants
QueryReify(InlineError<24>)
The left-hand data argument appears to be malformed.
XReify(InlineError<16>)
The right-hand data argument appears to be malformed.
UnequalLengths(UnequalLengths)
Distance computation failed because the logical lengths of the two vectors differ.
If vector reificiation occurs successfully, then this should not be returned.