pub enum QueryDistanceError {
XReify(InlineError<16>),
UnequalLengths(UnequalLengths),
}Expand description
Errors that can occur while perfoming distance cacluations on opaque vectors.
Variants
XReify(InlineError<16>)
The right-hand data argument appears to be malformed.
UnequalLengths(UnequalLengths)
Distance computation failed because the logical lengths of the two vectors differ.