pub enum QueryLayout {
SameAsData,
FourBitTransposed,
ScalarQuantized,
FullPrecision,
}Expand description
The layout to use for the query in DistanceComputer and QueryComputer.
Variants
SameAsData
Use the same compression strategy as the data vectors.
This may result in slow compression if high bit-widths are used.
FourBitTransposed
Use 4-bits for the query vector using a bitwise transpose layout.
ScalarQuantized
Use scalar quantization for the query using the same number of bits per dimension as the dataset.
FullPrecision
Use f32 to encode the query.