try_deserialize

Function try_deserialize 

pub fn try_deserialize<O, A>(
    data: &[u8],
    alloc: A,
) -> Result<Poly<dyn Quantizer<O>, A>, DeserializationError>
where O: Allocator + UnwindSafe + Send + Sync + 'static, A: Allocator + RefUnwindSafe + Send + Sync + 'static,
Available on crate feature flatbuffers only.
Attempt to deserialize a spherical::Quantizer flatbuffer into one of the concrete implementations of Quantizer.

This function guarantees that the returned Poly is the first object allocated through alloc.