DistanceComputer

Struct DistanceComputer 

pub struct DistanceComputer<A = GlobalAllocator>
where A: AllocatorCore,
{ /* private fields */ }
An opaque DistanceFunction for the Quantizer trait object.

Note

Left-hand arguments must be Opaque slices compressed using Quantizer::compress_query using Self::layout.

Right-hand arguments must be Opaque slices compressed using Quantizer::compress.

Otherwise, distance computations may return garbage values or panic.

Implementations

Source

impl<A> DistanceComputer<A>
where A: AllocatorCore,

pub fn layout(&self) -> QueryLayout

Report the layout used by the query computer.

pub fn into_inner(self) -> Poly<dyn DynDistanceComputer, A>