Impl

Struct Impl 

pub struct Impl<const NBITS: usize, A = GlobalAllocator>
where A: Allocator,
{ /* private fields */ }
Implementation for Quantizer specializing on the number of bits used for data compression.

Implementations

Source

impl<const NBITS: usize, A: Allocator> Impl<NBITS, A>

pub fn new(quantizer: SphericalQuantizer<A>) -> Result<Self, AllocatorError>
where Self: Constructible<A>,

Construct a new plan around quantizer providing distance computers for metric.

pub fn quantizer(&self) -> &SphericalQuantizer<A>

Return the underlying SphericalQuantizer.

pub fn supports(layout: QueryLayout) -> bool

Return true if this plan supports layout for query computers.

Otherwise, return false.