pub struct QueryBufferDescription { /* private fields */ }Expand description
A description of the buffer size (in bytes) and alignment required for a compressed query.
Implementations
Sourceimpl QueryBufferDescription
impl QueryBufferDescription
Sourcepub fn new(size: usize, align: PowerOfTwo) -> Self
pub fn new(size: usize, align: PowerOfTwo) -> Self
Construct a new QueryBufferDescription
Sourcepub fn bytes(&self) -> usize
pub fn bytes(&self) -> usize
Return the number of bytes needed in a buffer for a compressed query.
Sourcepub fn align(&self) -> PowerOfTwo
pub fn align(&self) -> PowerOfTwo
Return the necessary alignment of the base pointer for a query buffer.