QueryComputer

Struct QueryComputer 

pub struct QueryComputer<A = GlobalAllocator>
where A: AllocatorCore,
{ /* private fields */ }
An opaque PreprocessedDistanceFunction for the Quantizer trait object.

Note

This is only valid to call on Opaque slices compressed by the same Quantizer that created the computer.

Otherwise, distance computations may return garbage values or panic.

Implementations

Source

impl<A> QueryComputer<A>
where A: AllocatorCore,

pub fn layout(&self) -> QueryLayout

Report the layout used by the query computer.

pub fn into_inner(self) -> Poly<dyn DynQueryComputer, A>

This is a temporary function until custom allocator support fully comes on line.