#[repr(C)]pub struct DataMeta {
pub inner_product_correction: f16,
pub metric_specific: f16,
pub bit_sum: u16,
}Expand description
Metadata for correcting quantization for computing distances among quant vectors.
Fields
inner_product_correction: f16This is the whole term
|X| * a / <x', x>and represents the entires correction factor for computing inner products on the representation
bx + bwhere bx is unsigned binary encoding of the vector and b (obtained from
Self::offset_term) is the compression offset.
metric_specific: f16A metric-specific correction term. Refer to the module level documentation to understand the implication of the terms outlined here.
| Squared L2 | |X|^2 |
| Inner Product | <X', C> |
bit_sum: u16Two times the sum of the ones in the binary representation of the transformed unit vector.
This is the term sum(bx) in the module level documentation.
Implementations
Sourceimpl DataMeta
impl DataMeta
Sourcepub fn new(
inner_product_correction: f32,
metric_specific: f32,
bit_sum: u32,
) -> Result<Self, DataMetaError>
pub fn new( inner_product_correction: f32, metric_specific: f32, bit_sum: u32, ) -> Result<Self, DataMetaError>
Construct a new metadata from components.
This will internally convert the f32 values to f16.
Sourcepub fn to_full<A>(self, arch: A) -> DataMetaF32where
A: Architecture,
pub fn to_full<A>(self, arch: A) -> DataMetaF32where
A: Architecture,
Convert the values in self to their full precision representation for computation.