pub trait CompressInto<From, To> {
type Error: Error + 'static + Send + Sync;
type Output: Debug + Send + Sync;
// Required method
fn compress_into(
&self,
from: From,
to: To,
) -> Result<Self::Output, Self::Error>;
}Expand description
A trait implemented by a quantizer, indicating that it is capable of quantizing the
contents oF From into To.
Required Associated Types
Required Methods
Sourcefn compress_into(&self, from: From, to: To) -> Result<Self::Output, Self::Error>
fn compress_into(&self, from: From, to: To) -> Result<Self::Output, Self::Error>
Compress the data in From into To.
If an error is encountered, To must be left in a valid but undefined state.