pub struct ChunkOffsetsBase<T>{ /* private fields */ }Expand description
A wrapper class for PQ chunk offsets.
Upon construction, this class guarantees that the underlying chunk offset plan is valid.
A valid PQ chunk offset plan records the starting offsets of each chunk such that chunk
i of a slice x can be accessed using x[offsets[i]..offsets[i+1]].
In particular, a valid PQ chunk offset plan has the following properties:
- It has a length of at least 2.
- Its first entry is 0.
- For
iin0..offsets.len(),offsets[i] < offsets[i+1].
Implementations
Sourceimpl<T> ChunkOffsetsBase<T>
impl<T> ChunkOffsetsBase<T>
Sourcepub fn new(offsets: T) -> Result<Self, ChunkOffsetError>
pub fn new(offsets: T) -> Result<Self, ChunkOffsetError>
Construct a new ChunkOffset from a raw slice.
Returns an error if:
- The length of
offsetsis less than 2. - The entries in
offsetsare not strictly increasing.
Sourcepub fn len(&self) -> usize
pub fn len(&self) -> usize
Return the number of chunks associated with this mapping.
This will be one-less than the length of the provided slice.
Sourcepub fn dim(&self) -> usize
pub fn dim(&self) -> usize
Return the dimensionality of the vector data associated with this chunking schema.
Sourcepub fn dim_nonzero(&self) -> NonZeroUsize
pub fn dim_nonzero(&self) -> NonZeroUsize
Return the dimensionality of the vector data associated with this chunking schema.
By class invariant, the dimensionality must be nonzero, and this expressed in the retuen type.
This method cannot fail and will not panic.
Sourcepub fn at(&self, i: usize) -> Range<usize>
pub fn at(&self, i: usize) -> Range<usize>
Return a range containing the start and one-past-the-end indices for chunk i.
Panics
Panics if i >= self.len().
Sourcepub fn as_view(&self) -> ChunkOffsetsView<'_>
pub fn as_view(&self) -> ChunkOffsetsView<'_>
Return self as a view.
Sourcepub fn to_owned(&self) -> ChunkOffsets
pub fn to_owned(&self) -> ChunkOffsets
Return a 'static copy of self.