ChunkViewImpl

Struct ChunkViewImpl 

pub struct ChunkViewImpl<'a, T>
where T: DenseData,
{ /* private fields */ }
A view over a slice that partitions the slice into chunks corresponding to a valid PQ chunking configuration.

This class maintains the invariant that the provided chunking configuration if valid and that the data being partitioned has the correct length.

Implementations

Source

impl<'a, T> ChunkViewImpl<'a, T>
where T: DenseData,

pub fn new<U>( data: U, offsets: ChunkOffsetsView<'a>, ) -> Result<Self, ChunkViewError>
where T: From<U>,

Construct a new ChunkView.

Returns an error if data.len() != offsets.dim().

pub fn len(&self) -> usize

Return the number of partitions in the chunking view.

pub fn is_empty(&self) -> bool