TargetDim

Enum TargetDim 

pub enum TargetDim {
    Same,
    Natural,
    Override(NonZeroUsize),
}
Transformations possess the ability to keep dimensionality the same, increase it, or decrease it.

This struct enables the caller to communicate the desired behavior upon transform construction.

Variants



Same

Keep the output dimensionality the same as the input dimensionality.

Note

When the input dimensionality is less than the “natural” dimensionality ( see Self::Natural, post-transformed sampling may be invoked where only a subset of the transformed vector’s dimensions are retained.

For low dimensional embeddings, this sampling may result in high norm variance and poor recall.



Natural

Use the “natural” dimensionality for the output.

This allows transformations like PaddingHadamard to increase the dimensionality to the next power of two if needed. This will usually provide better accuracy than Self::Same but may result in a worse compression ratio.



Override(NonZeroUsize)

Set a hard value for the output dimensionality.

This may result in arbitrary subsampling (see the note in TargetDim::Same or supersampling (zero padding the pretransformed vector). Use with care.