RandomRotation

Struct RandomRotation 

pub struct RandomRotation { /* private fields */ }
Available on crate feature linalg only.
A distance-perserving transformation from N-dimensions to N-dimensions.

This struct materializes a full NxN transformation matrix and mechanically applies transformations via matrix multiplication.

Implementations

Source

impl RandomRotation

pub fn new<R>(dim: NonZeroUsize, target_dim: TargetDim, rng: &mut R) -> Self
where R: Rng + ?Sized,

Construct a new RandomRotation that transforms input vectors of dimension dim.

The parameter rng is used to randomly sample the transformation matrix.

The following dimensionalities will be configured depending on the value of target:

  • TargetDim::Same
    • self.input_dim() == dim.get()
    • self.output_dim() == dim.get()
  • TargetDim::Natural
    • self.input_dim() == dim.get()
    • self.output_dim() == dim.get()
  • TargetDim::Override
    • self.input_dim() == dim.get()
    • self.output_dim(): The value provided by the override.

Sub-sampling occurs if self.output_dim() is less than self.input_dim().

pub fn input_dim(&self) -> usize

Return the input dimension for the transformation.

pub fn output_dim(&self) -> usize

Return the output dimension for the transformation.

pub fn preserves_norms(&self) -> bool

Return whether or not the transform preserves norms.

For this transform, norms are not preserved when the output dimensionality is less than the input dimensionality.

pub fn transform_into( &self, dst: &mut [f32], src: &[f32], ) -> Result<(), TransformFailed>

Perform the transformation of the src vector into the dst vector.

Errors

Returns an error if

  • src.len() != self.input_dim().
  • dst.len() != self.output_dim().