Macros
Structs
- Aligned
Allocator - An
AllocatorCorethat allocates memory aligned to at least a specified alignment. - Allocator
Error - Indicate that an allocation error has occurred.
- Bump
Allocator - An
AllocatorCorethat pre-allocates a large buffer of memory and then satisfies allocation requests from that buffer. - Global
Allocator - A handle to Rust’s global allocator. This type does not support allocations of size 0.
- NotPower
OfTwo - Poly
- An owning pointer type like
std::Boxthat supports custom allocators. - Scoped
Allocator - A dynamic wrapper around an
AllocatorCorethat provides the guarantee that all allocated object are tied to a given scope.
Enums
- Compound
Error - Error type returned from
Poly::new_with.
Traits
- Allocator
- A dynamic memory allocator for use with
crate::alloc::Poly. - Allocator
Core - A dynamic memory allocator for use with
crate::alloc::Poly. - Trusted
Iter - A local marker type for iterators with a trusted length.