pub struct AlignedAllocator { /* private fields */ }Expand description
An AllocatorCore that allocates memory aligned to at least a specified alignment.
This can be useful for large allocations that need a predictable base alignment.
Implementations