pub struct ScopedAllocator<'a> { /* private fields */ }Expand description
A dynamic wrapper around an AllocatorCore that provides the guarantee that all
allocated object are tied to a given scope.
Additionally, this can allow the use of an allocator that is not Clone in contexts
where a clonable allocator is needed (provided the scoping limitations are acceptable).
Implementations
Sourceimpl<'a> ScopedAllocator<'a>
impl<'a> ScopedAllocator<'a>
Sourcepub const fn new<T>(allocator: &'a T) -> Selfwhere
T: AllocatorCore + Debug,
pub const fn new<T>(allocator: &'a T) -> Selfwhere
T: AllocatorCore + Debug,
Construct a new ScopedAllocator around the provided allocator.
Sourceimpl ScopedAllocator<'static>
impl ScopedAllocator<'static>
Sourcepub const fn global() -> Self
pub const fn global() -> Self
A convenience method for construcing a ScopedAllocator around the GlobalAllocator
for cases where a more specialized allocator is not needed.