pub unsafe trait TrustedIter: ExactSizeIterator { }Expand description
A local marker type for iterators with a trusted length.
Safety
Implementation must ensure that the implementation of ExactSizeIterator is such that
that unsafe code can rely on the returned value.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementations on Foreign Types
impl TrustedIter for IndexVecIntoIter
impl TrustedIter for Range<usize>
impl<'a, I, T> TrustedIter for Cloned<I>
impl<'a, I, T> TrustedIter for Copied<I>
impl<'a, T> TrustedIter for VectorIter<'a, T>where
T: Follow<'a>,
Available on crate feature
flatbuffers only.