Module bits

Module bits 

Utilities for working with packed 1-bit to 8-bit integers (inclusive).

Modules

distances
Low-level functions

Structs

Binary
A 1-bit binary quantization mapping -1 to 0 and 1 to 1.
BitSliceBase
A generalized representation for packed small bit integer encodings over a contiguous span of memory.
BitTranspose
A layout specialized for performing multi-bit operations with 1-bit scalar quantization.
ConstructionError
Dense
The identity permutation strategy.
Dynamic
A type for dynamically sized slices.
EncodingError
IndexOutOfBounds
MutSlicePtr
A mutable pointer with an associated lifetime.
SlicePtr
A constant pointer with an associated lifetime.
Static
A type for statically sized slices (slices with a length known at compile-time).
Unsigned
Storage unsigned integers in slices.

Enums

GetError
SetError

Traits

AsMutPtr
Obtain a mutable base pointer for a slice of data.
AsPtr
Obtain a constant base pointer for a slice of data.
Length
A trait allowing a type to yield a value.
PermutationStrategy
A enable the dimensions within a BitSlice to be permuted in an arbitrary way.
Representation
Representation of NBITS bit numbers in the associated domain.

Type Aliases

BitSlice
The layout for N-bit integers that references a raw underlying slice.
BoxedBitSlice
The layout for N-bit integers that own the underlying slice.
MutBitSlice
The layout for N-bit integers that mutable references a raw underlying slice.