BitTranspose

Struct BitTranspose 

pub struct BitTranspose;
A layout specialized for performing multi-bit operations with 1-bit scalar quantization.

The layout provided by this struct is as follows. Assume we are compressing N bit data. Then, the store the data in blocks of 64 * N bits (where 64 comes from the native CPU word size).

Each block can contain 64 values, stored in N 64-bit words. The 0th bit of each value is stored in word 0, the 1st bit is stored in word 1, etc.

Partially Filled Blocks

This strategy always requests data in blocks. For partially filled blocks, the lower bits in the last block will be used.