pub type MathematicalResult<T> = Result<MathematicalValue<T>, UnequalLengths>;Expand description
Upcoming return type for distance functions to allow graceful failure instead of panicking when incorrect dimensions are provided.
Aliased Type
pub enum MathematicalResult<T> {
Ok(MathematicalValue<T>),
Err(UnequalLengths),
}Variants
Ok(MathematicalValue<T>)
Contains the success value
Err(UnequalLengths)
Contains the error value