pub struct VectorBase<const NBITS: usize, Repr, Ptr, T, Perm = Dense>{ /* private fields */ }Expand description
A wrapper for BitSliceBase that provides the addition of arbitrary metadata.
Examples
The VectorBase has several named variants that are commonly used:
Vector: An owning, independently allocatedVectorBase.VectorMut: A mutable, reference-like type to aVectorBase.VectorRef: A const, reference-like type to aVectorBase.
use diskann_quantization::{
meta::{Vector, VectorMut, VectorRef},
bits::Unsigned,
};
use diskann_utils::{Reborrow, ReborrowMut};
#[derive(Debug, Default, Clone, Copy, PartialEq)]
struct Metadata {
value: f32,
}
// Create a new heap-allocated Vector for 4-bit compressions capable of
// holding 3 elements.
//
// In this case, the associated m
let mut v = Vector::<4, Unsigned, Metadata>::new_boxed(3);
// We can inspect the underlying bitslice.
let bitslice = v.vector();
assert_eq!(bitslice.get(0).unwrap(), 0);
assert_eq!(bitslice.get(1).unwrap(), 0);
assert_eq!(v.meta(), Metadata::default(), "expected default metadata value");
// If we want, we can mutably borrow the bitslice and mutate its components.
let mut bitslice = v.vector_mut();
bitslice.set(0, 1).unwrap();
bitslice.set(1, 2).unwrap();
bitslice.set(2, 3).unwrap();
assert!(bitslice.set(3, 4).is_err(), "out-of-bounds access");
// Get the underlying pointer for comparison.
let ptr = bitslice.as_ptr();
// Vectors can be converted to a generalized reference.
let mut v_ref = v.reborrow_mut();
// The generalized reference preserves the underlying pointer.
assert_eq!(v_ref.vector().as_ptr(), ptr);
let mut bitslice = v_ref.vector_mut();
bitslice.set(0, 10).unwrap();
// Setting the underlying compensation will be visible in the original allocation.
v_ref.set_meta(Metadata { value: 10.5 });
// Check that the changes are visible.
assert_eq!(v.meta().value, 10.5);
assert_eq!(v.vector().get(0).unwrap(), 10);
// Finally, the immutable ref also maintains pointer compatibility.
let v_ref = v.reborrow();
assert_eq!(v_ref.vector().as_ptr(), ptr);Constructing a VectorMut From Components
The following example shows how to assemble a VectorMut from raw memory.
use diskann_quantization::{bits::{Unsigned, MutBitSlice}, meta::VectorMut};
// Start with 2 bytes of memory. We will impose a 4-bit scalar quantization on top of
// these 2 bytes.
let mut data = vec![0u8; 2];
let mut metadata: f32 = 0.0;
{
// First, we need to construct a bit-slice over the data.
// This will check that it is sized properly for 4, 4-bit values.
let mut slice = MutBitSlice::<4, Unsigned>::new(data.as_mut_slice(), 4).unwrap();
// Next, we construct the `VectorMut`.
let mut v = VectorMut::new(slice, &mut metadata);
// Through `v`, we can set all the components in `slice` and the compensation.
v.set_meta(123.4);
let mut from_v = v.vector_mut();
from_v.set(0, 1).unwrap();
from_v.set(1, 2).unwrap();
from_v.set(2, 3).unwrap();
from_v.set(3, 4).unwrap();
}
// Now we can check that the changes made internally are visible.
assert_eq!(&data, &[0x21, 0x43]);
assert_eq!(metadata, 123.4);Canonical Layout
When the metadata type T is
bytemuck::Pod, VectorRef
and VectorMut support layout canonicalization, where a raw slice can be used as the
backing store for such vectors, enabling inline storage.
There are two supported schems for the canonical layout, depending on whether the metadata is located at the beginning of the slice or at the end of the slice.
If the metadata is at the front, then the layout consists of a slice &[u8] where the
first std::mem::size_of::<T>() bytes are the metadata and the remainder compose the
BitSlice codes.
If the metadata is at the back, , then the layout consists of a slice &[u8] where the
last std::mem::size_of::<T>() bytes are the metadata and the prefix is the
BitSlice codes.
The canonical layout needs the following properties:
T: bytemuck::Pod: For safely storing and retrieving.- The length for a vector with
Ndimensions must be equal to the value returne fromVector::canonical_bytes.
The following functions can be used to construct VectorBases from raw slices:
VectorRef::from_canonical_frontVectorRef::from_canonical_backVectorMut::from_canonical_front_mutVectorMut::from_canonical_back_mut
An example is shown below.
use diskann_quantization::{bits, meta::{Vector, VectorRef, VectorMut}};
type CVRef<'a, const NBITS: usize> = VectorRef<'a, NBITS, bits::Unsigned, f32>;
type MutCV<'a, const NBITS: usize> = VectorMut<'a, NBITS, bits::Unsigned, f32>;
let dim = 3;
// Since we don't control the alignment of the returned pointer, we need to oversize it.
let bytes = CVRef::<4>::canonical_bytes(dim);
let mut data: Box<[u8]> = (0..bytes).map(|_| u8::default()).collect();
// Construct a mutable compensated vector over the slice.
let mut mut_cv = MutCV::<4>::from_canonical_front_mut(&mut data, dim).unwrap();
mut_cv.set_meta(1.0);
let mut v = mut_cv.vector_mut();
v.set(0, 1).unwrap();
v.set(1, 2).unwrap();
v.set(2, 3).unwrap();
// Reconstruct a constant CompensatedVector.
let cv = CVRef::<4>::from_canonical_front(&data, dim).unwrap();
assert_eq!(cv.meta(), 1.0);
let v = cv.vector();
assert_eq!(v.get(0).unwrap(), 1);
assert_eq!(v.get(1).unwrap(), 2);
assert_eq!(v.get(2).unwrap(), 3);Implementations
Sourceimpl<const NBITS: usize, Repr, Ptr, T, Perm> VectorBase<NBITS, Repr, Ptr, T, Perm>
impl<const NBITS: usize, Repr, Ptr, T, Perm> VectorBase<NBITS, Repr, Ptr, T, Perm>
Sourcepub fn slice_bytes(count: usize) -> usize
pub fn slice_bytes(count: usize) -> usize
Return the number of bytes required for the underlying BitSlice.
Sourcepub fn canonical_bytes(count: usize) -> usizewhere
T: CopyRef,
T::Target: Pod,
pub fn canonical_bytes(count: usize) -> usizewhere
T: CopyRef,
T::Target: Pod,
Return the number of bytes required for the canonical representation of a
Vector.
See: VectorRef::from_canonical_back, VectorMut::from_canonical_back_mut.
Sourcepub fn new<M>(bits: BitSliceBase<NBITS, Repr, Ptr, Perm>, meta: M) -> Selfwhere
M: Into<T>,
pub fn new<M>(bits: BitSliceBase<NBITS, Repr, Ptr, Perm>, meta: M) -> Selfwhere
M: Into<T>,
Construct a new VectorBase over the bit-slice.
Sourcepub fn vector_mut(&mut self) -> MutBitSlice<'_, NBITS, Repr, Perm>where
Ptr: AsMutPtr,
pub fn vector_mut(&mut self) -> MutBitSlice<'_, NBITS, Repr, Perm>where
Ptr: AsMutPtr,
Mutably borrow the integer compressed vector.
Sourceimpl<const NBITS: usize, Repr, Perm, T> VectorBase<NBITS, Repr, Poly<[u8], GlobalAllocator>, Owned<T>, Perm>
impl<const NBITS: usize, Repr, Perm, T> VectorBase<NBITS, Repr, Poly<[u8], GlobalAllocator>, Owned<T>, Perm>
Sourceimpl<const NBITS: usize, Repr, Perm, T, A> VectorBase<NBITS, Repr, Poly<[u8], A>, Owned<T>, Perm>
impl<const NBITS: usize, Repr, Perm, T, A> VectorBase<NBITS, Repr, Poly<[u8], A>, Owned<T>, Perm>
Sourcepub fn new_in(len: usize, allocator: A) -> Result<Self, AllocatorError>
pub fn new_in(len: usize, allocator: A) -> Result<Self, AllocatorError>
Create a new owned VectorBase with its metadata default initialized.
Sourceimpl<'a, const NBITS: usize, Repr, T, Perm> VectorBase<NBITS, Repr, SlicePtr<'a, u8>, Ref<'a, T>, Perm>
impl<'a, const NBITS: usize, Repr, T, Perm> VectorBase<NBITS, Repr, SlicePtr<'a, u8>, Ref<'a, T>, Perm>
Sourcepub fn from_canonical_front(
data: &'a [u8],
dim: usize,
) -> Result<Self, NotCanonical>
pub fn from_canonical_front( data: &'a [u8], dim: usize, ) -> Result<Self, NotCanonical>
Construct an instance of Self viewing data as the canonical layout for a vector.
The canonical layout is as follows:
std::mem::size_of::<T>()for the metadata coefficient.Self::slice_bytes(dim)for the underlying bit-slice.
Returns an error if data.len() != Self::canonical_bytes`.
Sourcepub fn from_canonical_back(
data: &'a [u8],
dim: usize,
) -> Result<Self, NotCanonical>
pub fn from_canonical_back( data: &'a [u8], dim: usize, ) -> Result<Self, NotCanonical>
Construct an instance of Self viewing data as the canonical layout for a vector.
The back canonical layout is as follows:
Self::slice_bytes(dim)for the underlying bit-slice.std::mem::size_of::<T>()for the metadata coefficient.
Returns an error if data.len() != Self::canonical_bytes`.
Sourcepub unsafe fn from_canonical_unchecked(data: &'a [u8], dim: usize) -> Self
pub unsafe fn from_canonical_unchecked(data: &'a [u8], dim: usize) -> Self
Construct a VectorRef from the raw data.
Safety
data.len()must be equal toSelf::canonical_bytes(dim).
This invariant is checked in debug builds and will panic if not satisfied.
Sourcepub unsafe fn from_canonical_back_unchecked(data: &'a [u8], dim: usize) -> Self
pub unsafe fn from_canonical_back_unchecked(data: &'a [u8], dim: usize) -> Self
Construct a VectorRef from the raw data.
Safety
data.len()must be equal toSelf::canonical_bytes(dim).
This invariant is checked in debug builds and will panic if not satisfied.
Sourceimpl<'a, const NBITS: usize, Repr, T, Perm> VectorBase<NBITS, Repr, MutSlicePtr<'a, u8>, Mut<'a, T>, Perm>
impl<'a, const NBITS: usize, Repr, T, Perm> VectorBase<NBITS, Repr, MutSlicePtr<'a, u8>, Mut<'a, T>, Perm>
Sourcepub fn from_canonical_front_mut(
data: &'a mut [u8],
dim: usize,
) -> Result<Self, NotCanonical>
pub fn from_canonical_front_mut( data: &'a mut [u8], dim: usize, ) -> Result<Self, NotCanonical>
Construct an instance of Self viewing data as the canonical layout for a vector.
The canonical layout is as follows:
std::mem::size_of::<T>()for the metadata coefficient.Self::slice_bytes(dim)for the underlying bit-slice.
Returns an error if data.len() != Self::canonical_bytes`.
Sourcepub unsafe fn from_canonical_front_mut_unchecked(
data: &'a mut [u8],
dim: usize,
) -> Self
pub unsafe fn from_canonical_front_mut_unchecked( data: &'a mut [u8], dim: usize, ) -> Self
Construct a VectorMut from the raw data.
Safety
data.len()must be equal toSelf::canonical_bytes(dim).
This invariant is checked in debug builds and will panic if not satisfied.
Sourcepub fn from_canonical_back_mut(
data: &'a mut [u8],
dim: usize,
) -> Result<Self, NotCanonical>
pub fn from_canonical_back_mut( data: &'a mut [u8], dim: usize, ) -> Result<Self, NotCanonical>
Construct an instance of Self viewing data as the canonical layout for a vector.
The back canonical layout is as follows:
Self::slice_bytes(dim)for the underlying bit-slice.std::mem::size_of::<T>()for the metadata coefficient.
Returns an error if data.len() != Self::canonical_bytes`.
Sourceimpl<const NBITS: usize> VectorBase<NBITS, Unsigned, SlicePtr<'_, u8>, Ref<'_, MinMaxCompensation>>where
Unsigned: Representation<NBITS>,
impl<const NBITS: usize> VectorBase<NBITS, Unsigned, SlicePtr<'_, u8>, Ref<'_, MinMaxCompensation>>where
Unsigned: Representation<NBITS>,
Sourcepub fn decompress_into(&self, dst: &mut [f32]) -> Result<(), DecompressError>
pub fn decompress_into(&self, dst: &mut [f32]) -> Result<(), DecompressError>
Decompresses a MinMax quantized vector back into its original floating-point representation.
This method reconstructs the original vector values using the stored quantization parameters
and the MinMax dequantization formula: x = x' * a + b and stores the result in dst
Arguments
dst- A mutable slice off32values where the decompressed data will be written. Must have the same length as the compressed vector.
Returns
Ok(())- On successful decompressionErr(DecompressError::LengthMismatch(src_len, dst_len))- If the destination slice length doesn’t match the compressed vector length