pub struct MinMaxMeta<const NBITS: usize> { /* private fields */ }Expand description
Metadata for MinMax quantized multi-vectors.
Stores the intrinsic dimension (output dimension after transform) which is
needed to interpret each row of the quantized data. The row stride in bytes
is computed as [Data::<NBITS>::canonical_bytes(intrinsic_dim)].
Implementations
Sourceimpl<const NBITS: usize> MinMaxMeta<NBITS>where
Unsigned: Representation<NBITS>,
impl<const NBITS: usize> MinMaxMeta<NBITS>where
Unsigned: Representation<NBITS>,
Sourcepub fn new(nrows: usize, intrinsic_dim: usize) -> Self
pub fn new(nrows: usize, intrinsic_dim: usize) -> Self
Creates new MinMax metadata with the given number of rows and intrinsic dimension.
Sourcepub fn intrinsic_dim(&self) -> usize
pub fn intrinsic_dim(&self) -> usize
Returns the intrinsic_dim
Sourcepub fn ncols(&self) -> usize
pub fn ncols(&self) -> usize
Returns the number of bytes from a canonical repr of a minmax quantized vector, see minmax::Data::canonical_bytes