pub struct MaxSim<'a> { /* private fields */ }Expand description
Computes per-query-vector maximum similarities to document vectors.
For each query vector qᵢ, finds the maximum similarity (minimum negated
inner product) to any document vector:
scores[i] = minⱼ -IP(qᵢ, dⱼ)Implements DistanceFnMut for various matrix types
(e.g., MatRef<Standard<f32>>).
Usage
- Create with
MaxSim::new, providing a mutable scores buffer. - Call
DistanceFnMut::evaluatewith query and document matrices. - Read results from the scores buffer.
Implementations
Sourceimpl<'a> MaxSim<'a>
impl<'a> MaxSim<'a>
Sourcepub fn new(scores: &'a mut [f32]) -> Result<Self, MaxSimError>
pub fn new(scores: &'a mut [f32]) -> Result<Self, MaxSimError>
Sourcepub fn scores_mut(&mut self) -> &mut [f32]
pub fn scores_mut(&mut self) -> &mut [f32]
Returns a mutable reference to the internal buffer of scores.
This is useful for implementations external to crate as well as optimized implementations to access the buffer if needed.