pub struct Mat<T: ReprOwned> { /* private fields */ }Expand description
An owning matrix that manages its own memory.
The matrix stores raw bytes interpreted according to representation type T.
Memory is automatically deallocated when the matrix is dropped.
Implementations
Sourceimpl<T: ReprOwned> Mat<T>
impl<T: ReprOwned> Mat<T>
Sourcepub fn new<U>(repr: T, init: U) -> Result<Self, <T as NewOwned<U>>::Error>where
T: NewOwned<U>,
pub fn new<U>(repr: T, init: U) -> Result<Self, <T as NewOwned<U>>::Error>where
T: NewOwned<U>,
Create a new matrix using init as the initializer.
Sourcepub fn num_vectors(&self) -> usize
pub fn num_vectors(&self) -> usize
Returns the number of rows (vectors) in the matrix.
Sourcepub fn get_row(&self, i: usize) -> Option<T::Row<'_>>
pub fn get_row(&self, i: usize) -> Option<T::Row<'_>>
Returns the ith row if i < self.num_vectors().
Sourcepub fn get_row_mut(&mut self, i: usize) -> Option<T::RowMut<'_>>
pub fn get_row_mut(&mut self, i: usize) -> Option<T::RowMut<'_>>
Returns the ith mutable row if i < self.num_vectors().
Sourcepub fn as_view_mut(&mut self) -> MatMut<'_, T>
pub fn as_view_mut(&mut self) -> MatMut<'_, T>
Returns a mutable view of the matrix.
Sourcepub fn as_raw_ptr(&self) -> *const u8
pub fn as_raw_ptr(&self) -> *const u8
Return the base pointer for the Mat.