pub struct MatRef<'a, T: Repr> { /* private fields */ }Expand description
Implementations
Sourceimpl<'a, T: Repr> MatRef<'a, T>
impl<'a, T: Repr> MatRef<'a, T>
Sourcepub fn new<U>(repr: T, data: &'a [U]) -> Result<Self, T::Error>where
T: NewRef<U>,
pub fn new<U>(repr: T, data: &'a [U]) -> Result<Self, T::Error>where
T: NewRef<U>,
Construct a new MatRef over data.
Sourcepub fn num_vectors(&self) -> usize
pub fn num_vectors(&self) -> usize
Returns the number of rows (vectors) in the matrix.
Sourcepub fn get_row(&self, i: usize) -> Option<T::Row<'_>>
pub fn get_row(&self, i: usize) -> Option<T::Row<'_>>
Returns an immutable reference to the i-th row, or None if out of bounds.
Sourcepub fn to_owned(&self) -> Mat<T>where
T: NewCloned,
pub fn to_owned(&self) -> Mat<T>where
T: NewCloned,
Return a Mat with the same contents as self.
Sourcepub unsafe fn from_raw_parts(repr: T, ptr: NonNull<u8>) -> Self
pub unsafe fn from_raw_parts(repr: T, ptr: NonNull<u8>) -> Self
Construct a new MatRef over the raw pointer and representation without performing
any validity checks.
Safety
Argument ptr must point to memory compatible with Repr::layout and pass any
validity checks required by T.
Sourcepub fn as_raw_ptr(&self) -> *const u8
pub fn as_raw_ptr(&self) -> *const u8
Return the base pointer for the MatRef.