PowerOfTwo

Struct PowerOfTwo 

pub struct PowerOfTwo(/* private fields */);

Implementations

Source

impl PowerOfTwo

pub const fn new(value: usize) -> Result<Self, NotPowerOfTwo>

Create a new PowerOfTwo if the given value is greater a power of two.

pub const fn next(value: usize) -> Option<Self>

Return the smallest power of two greater than or equal to value. If the next power of two is greater than usize::MAX, None is returned.

pub const unsafe fn new_unchecked(value: NonZeroUsize) -> Self

Create a new PowerOfTwo without checking whether the value is a power of two.

Safety

The value must be a power of two.

pub const fn into_inner(self) -> NonZeroUsize

Consume self and return the inner value.

pub const fn raw(self) -> usize

Consume self and return the inner value as a usize.

pub const fn from_align(layout: &Layout) -> Self

Construct self from the alignment in layout.

pub const fn alignment_of<T>() -> Self

Return the alignment of T as a power of two.

pub const fn arg_mod(self, lhs: usize) -> usize

Compute the operation lhs % self.

Note

The argument order of this function is reversed from the typical align_offset method in the standard library.

pub const fn arg_align_offset(self, lhs: usize) -> usize

Compute the amount x that would have to be added to lhs so the quantity lhs + x is a multiple of self.

Note

The argument order of this function is reversed from the typical align_offset method in the standard library.

pub const fn arg_checked_next_multiple_of(self, lhs: usize) -> Option<usize>

Calculate the smallest value greater than or equal to lhs that is a multiple of self. Return None if the operation would result in an overflow.