Product quantization training and compression.
Modules
Structs
- Basic
Table Base - A basic PQ table that stores the pivot table in the following dense, row-major form:
- Transposed
Table - A PQ table that stores the pivots for each chunk in a miniture block-transpose to
facilitate faster compression. The exact layout is not documented (as it is for the
BasicTable) because it may be subject to change.
Enums
Type Aliases
- Basic
Table - A
BasicTableBasethat owns its contents. - Basic
Table View - A
BasicTableBasethat references its contents. Construction of such a table will not result in a memory allocation.