The main export of this module is the dyn compatible Quantizer trait, which provides
a common interface for interacting with bit-width specific SphericalQuantizers,
compressing vectors, and computing distances between compressed vectors.
A DistanceFunction containing scaling parameters to enable distance the Cosine
distance function over vectors belonging to the same quantization space.
A DistanceFunction containing scaling parameters to enable distance the SquaredL2
distance function over CompensatedVectors belonging to the same quantization space.
A DistanceFunction containing scaling parameters to enable distance the SquaredL2
distance function over CompensatedVectors belonging to the same quantization space.
Pre-scaling selector for spherical quantization training. Pre-scaling adjusts the
dynamic range of the data (usually decreasing it uniformly) to keep the correction terms
within the range expressible by 16-bit floating point numbers.