pub struct DistanceComputer<A = GlobalAllocator>where
A: AllocatorCore,{ /* private fields */ }Expand description
An opaque DistanceFunction for the Quantizer trait object.
Note
Left-hand arguments must be Opaque slices compressed using
Quantizer::compress_query using Self::layout.
Right-hand arguments must be Opaque slices compressed using Quantizer::compress.
Otherwise, distance computations may return garbage values or panic.
Implementations
Sourceimpl<A> DistanceComputer<A>where
A: AllocatorCore,
impl<A> DistanceComputer<A>where
A: AllocatorCore,
Sourcepub fn layout(&self) -> QueryLayout
pub fn layout(&self) -> QueryLayout
Report the layout used by the query computer.
pub fn into_inner(self) -> Poly<dyn DynDistanceComputer, A>