pub struct Impl<const NBITS: usize, A = GlobalAllocator>where
A: Allocator,{ /* private fields */ }Expand description
Implementation for Quantizer specializing on the number of bits used for data
compression.
Implementations
Sourceimpl<const NBITS: usize, A: Allocator> Impl<NBITS, A>
impl<const NBITS: usize, A: Allocator> Impl<NBITS, A>
Sourcepub fn new(quantizer: SphericalQuantizer<A>) -> Result<Self, AllocatorError>where
Self: Constructible<A>,
pub fn new(quantizer: SphericalQuantizer<A>) -> Result<Self, AllocatorError>where
Self: Constructible<A>,
Construct a new plan around quantizer providing distance computers for metric.
Sourcepub fn quantizer(&self) -> &SphericalQuantizer<A>
pub fn quantizer(&self) -> &SphericalQuantizer<A>
Return the underlying SphericalQuantizer.
Sourcepub fn supports(layout: QueryLayout) -> bool
pub fn supports(layout: QueryLayout) -> bool
Return true if this plan supports layout for query computers.
Otherwise, return false.