pub struct CompensatedCosine { /* private fields */ }Expand description
A DistanceFunction containing scaling parameters to enable distance the Cosine
distance function over vectors belonging to the same quantization space.
This distance function works by assuming input vectors were normalized prior to
compression and therefore cosine may be computed by delegating to inner product
computations. The crate::spherical::SphericalQuantizer will ensure this
pre-normalization when constructed with SupportedMetric::Cosine.
Implementations
Sourceimpl CompensatedCosine
impl CompensatedCosine
Sourcepub fn new(inner: CompensatedIP) -> Self
pub fn new(inner: CompensatedIP) -> Self
Construct a new CompensatedCosine around the CompensatedIP.