#[repr(C)]pub struct QueryMeta {
pub inner_product_correction: f32,
pub bit_sum: f32,
pub offset: f32,
pub metric_specific: f32,
}Expand description
Scalar quantization correction factors for computing distances between scalar quantized queries and spherically quantized data elements.
Computing the distance between a query and a data vector uses the same forumla derived in the module level documentation.
The one difference is that the query must explicitly carry the “offset” term as it cannot be derived from the number of bits used for the compression.
Fields
inner_product_correction: f32The value with which to scale the bit-level inner product with the 1-bit data data vectors.
bit_sum: f32Scaling factor for the DataMeta::twice_contraction. Applied separately to
still allow 1-bit vectors using DataMeta to compute distances with eachother
efficiently.
offset: f32The query-specific offset, taking into account the scaling factor for the query as well as its minimum value. See the struct-level documentation for an explanation.
metric_specific: f32The corresponding metric specific term as DataMeta.