ChunkOffsetsBase

Struct ChunkOffsetsBase 

pub struct ChunkOffsetsBase<T>
where T: DenseData<Elem = usize>,
{ /* private fields */ }
A wrapper class for PQ chunk offsets.

Upon construction, this class guarantees that the underlying chunk offset plan is valid. A valid PQ chunk offset plan records the starting offsets of each chunk such that chunk i of a slice x can be accessed using x[offsets[i]..offsets[i+1]].

In particular, a valid PQ chunk offset plan has the following properties:

  • It has a length of at least 2.
  • Its first entry is 0.
  • For i in 0..offsets.len(), offsets[i] < offsets[i+1].

Implementations

Source

impl<T> ChunkOffsetsBase<T>
where T: DenseData<Elem = usize>,

pub fn new(offsets: T) -> Result<Self, ChunkOffsetError>

Construct a new ChunkOffset from a raw slice.

Returns an error if:

  • The length of offsets is less than 2.
  • The entries in offsets are not strictly increasing.
pub fn len(&self) -> usize

Return the number of chunks associated with this mapping.

This will be one-less than the length of the provided slice.

pub fn is_empty(&self) -> bool

Return whether the offsets are empty.

pub fn dim(&self) -> usize

Return the dimensionality of the vector data associated with this chunking schema.

pub fn dim_nonzero(&self) -> NonZeroUsize

Return the dimensionality of the vector data associated with this chunking schema.

By class invariant, the dimensionality must be nonzero, and this expressed in the retuen type.

This method cannot fail and will not panic.

pub fn at(&self, i: usize) -> Range<usize>

Return a range containing the start and one-past-the-end indices for chunk i.

Panics

Panics if i >= self.len().

pub fn as_view(&self) -> ChunkOffsetsView<'_>

Return self as a view.

pub fn to_owned(&self) -> ChunkOffsets

Return a 'static copy of self.

pub fn as_slice(&self) -> &[usize]

Return the underlying data as a slice.